@@ -1,12 +1,12 @@
 /*
- * $Id: atexit.c 6358 1998-09-01 17:32:40Z tmap $
+ * $Id: atexit.c 19905 2015-07-01 00:25:30Z ksmith $
  */
 
 /*LINTLIBRARY*/
 
 
 #include "udposix.h"
-#include "stdlib.h"
+#include <stdlib.h>
 
 
     int
