@@ -75,7 +75,7 @@
 #   define	malloc(n)	(lint_malloc((n)), 0)
 #else
     static char afsid[]	= "$__Header$";
-    static char rcsid[]	= "$Id: xopws.c 6517 1999-03-09 20:08:23Z sirott $";
+    static char rcsid[]	= "$Id: xopws.c 20213 2015-08-17 22:09:56Z ksmith $";
 #endif
 
 static int      InitDefaults();
@@ -156,7 +156,8 @@ xXgksOpenWs(wk)
 /* *sh* 16-sep-94: bypass this code for invisible, unmapped windows */
 		if ( map_that_window ) {
 		  XMapWindow(wk->dpy, wk->win);
-		  XWindowEvent(wk->dpy, wk->win, ExposureMask, &xev);
+		  /* Exposure event not always caught or created? */
+		  /* XWindowEvent(wk->dpy, wk->win, ExposureMask, &xev); */
 		  XSync(wk->dpy, 0);
 		}
 
@@ -165,6 +166,9 @@ xXgksOpenWs(wk)
 		wk->wbound.x = WinAtt.width;
 		wk->wbound.y = WinAtt.height;
 
+		/* Clear the window; use XClearArea since we know the width, height */
+		XClearArea(wk->dpy, wk->win, 0, 0, WinAtt.width, WinAtt.height, 0);
+
 		/* Update open-workstation table. */
 		(void) UpdateOpenWSTable(wk);
 
